from flask import Flask, jsonify
from http.server import HTTPServer, BaseHTTPRequestHandler
from pymongo import MongoClient
from pymongo.server_api import ServerApi
from consts import DB_NAME, COLLECTION, MONGO_URI, MONGO_CERT_PATH, SSL_CERT_PATH, SSL_KEY_PATH, HOST, PORT
from map.get_markers import get_markers
import atexit
import ssl

app = Flask(__name__)

# Initialise MongoDB connection
try:
    client = MongoClient(MONGO_URI,
                         tls=True,
                         tlsCertificateKeyFile=MONGO_CERT_PATH,
                         server_api=ServerApi('1'))

    db = client[DB_NAME]
    collection = db[COLLECTION]
    print("Connected to MongoDB")
except Exception as e:
    print(f"Failed to connect to MongoDB: {e}")

# Define HTTP class
class IceHTTP(BaseHTTPRequestHandler):
    def do_GET(self):
        if self.path == '/':
            self.send_response(200)
            self.send_header("Content-type", "text/plain")
            self.end_headers()
            self.wfile.write(b"Root path hit!")
        elif self.path == '/update_map':
            # Send the response with a status code (e.g., 200 for success)
            self.send_response(200)
            self.send_header("Content-type", "application/json")
            self.end_headers()

            # Write the JSON data directly to the response
            markers_data = get_markers()
            self.wfile.write(markers_data.encode('utf-8'))

        elif self.path == '/test_endpoint':
            self.send_response(200)
            self.send_header("Content-type", "text/plain")
            self.end_headers()
            self.wfile.write(b"Test path hit!")


# Start a server on port 8443 using defined HTTP class
if __name__ == "__main__":
    print("Starting server on port ", PORT)
    try:
        # Load SSL certificate and private key
        ssl_context = ssl.SSLContext(ssl.PROTOCOL_TLS_SERVER)
        ssl_context.load_cert_chain(SSL_CERT_PATH, SSL_KEY_PATH)

        # Create HTTP server with SSL support
        server = HTTPServer((HOST, PORT), IceHTTP)
        server.socket = ssl_context.wrap_socket(server.socket, server_side=True)

        print("Server running on port ", PORT)
        server.serve_forever()
    except Exception as e:
        print(f"Failed to start server on port {PORT}: {e}")


def on_termination():
    print("Server is terminating.")

# Register the function to be called on exit
atexit.register(on_termination)
       